%define releasebase 5
%define nbase fftw
%define prefix /opt/%{nbase}2/%{version}/%{compilertag}/%{mpitag}
%define modulepath /opt/Modules/modulefiles/

#Use as deafults if no command line inputs
%define as_limulus 1
%define use_gnu4  1
#  NOTE: MPI's are mutually exclusive, only set one,
#        if not using command line options
#        Right now they only work with gnu 
#        gcc3.x and gcc4.x, it will work with pgi and mpichgm-pgi
#        other compilers and have not been thoroughly tested or configured
%define use_mpich2 0
%define use_ompi  1 
# allow --with[out] <feature> at rpm command line build
%{?_with_basement: %{expand: %%define as_basement 1}}
%{?_with_limulus: %{expand: %%define as_limulus 1}}
#possible Compilers
%{?_with_gnu3: %{expand: %%define use_gnu3 1}}
%{?_with_gnu4: %{expand: %%define use_gnu4 1}}
%{?_with_pgi: %{expand: %%define use_pgi 1}}
%{?_with_intel: %{expand: %%define use_intel 1}}
%{?_with_pathsc: %{expand: %%define use_pathsc 1}}
# possible MPI's
%{?_with_lam: %{expand: %%define use_lam 1}}
%{?_with_mpich: %{expand: %%define use_mpich 1}}
%{?_with_mpich2: %{expand: %%define use_mpich2 1}}
%{?_with_mpich2omx: %{expand: %%define use_mpich2omx 1}}
%{?_with_ompi: %{expand: %%define use_ompi 1}}
#pick the target
%if "1" == "%{as_basement}"
%define targetbase Basement
%else
%define targetbase Limulus
%endif
# pick the compiler
%if "1" == "%{use_gnu4}"
%define compilertag gnu4
%define openMP  --without-openmp
%ifarch i386
%define fftw2hacks   --enable-i386-hacks 
%endif
%ifarch X86_64
%define fftw2hacks --disable-i386-hacks  
%endif
%endif
%if "1" == "%{use_pgi}"
%define compilertag pgi
%define fftw2hacks --disable-i386-hacks
%endif
%if "1" == "%{use_intel}"
%define compilertag intel
%endif
%if "1" == "%{use_pathsc}"
%define compilertag pathsc
%endif
%if "1" == "%{use_gnu3}"
%define compilertag gnu3
%define openMP  --without-openmp
%ifarch i386
%define fftw2hacks  --enable-i386-hacks 
%endif
%ifarch X86_64
%define fftw2hacks --disable-i386-hacks  
%endif
%endif
# pick the MPI
%if "1" == "%{use_ompi}"
%define mpitag openmpi 
%endif
%if "1" == "%{use_mpich2omx}"
%define mpitag mpich2-omx
%endif
%if "1" == "%{use_mpich2}"
%define mpitag mpich2
%endif


Name: %{nbase}2-%{mpitag}-%{compilertag}
Summary: Fast Fourier Transform library
Version: 2.1.5
Group: %{targetbase}/Libraries
Release: %{releasebase}.%{targetbase}
License: GPL
URL: http://www.fftw.org/
Packager: deadline@basement-supercomputing.com
Vendor: Basement-Supercomputing.com
Source: %{nbase}-%{version}.tar.gz
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root
Requires: modules  fftw2-%{compilertag} %{mpitag}-%{compilertag}
BuildRequires: modules  %{mpitag}-%{compilertag}



%description
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This package only contains the MPI libraries for fftw-2

%prep
%setup -q -c  %{nbase}-%{version}
mv %{nbase}-%{version} single
cp -a single double


%build
#we need modules to build this
rpm -qf /opt/Modules/bin/modulecmd >& /dev/null
if [ $? -ne 0 ]; then
  echo "This RPM requries the BSC Modules package to build"
fi
. /etc/profile.d/modules.sh

%if "1" == "%{use_intel}"
echo "Intel Comp not implemented"
exit
%endif
%if "1" == "%{use_pgi}"
  	module load pgi62
	  export CC=pgcc
    export F77=pgf77
    export OPENMP_CFLAGS='-mp'
    export CFLAGS='-fastsse -O3 '
    export FFLAGS='-fastsse -O3 '
%endif
%if "1" == "%{use_pathsc}"
echo "Path Comp not implemented"
exit
%endif

# GNU4
%if "1" == "%{use_gnu4}"
        export CC=gcc
        export CFLAGS='-O2 -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE'
        export F77=gfortran
        export FFLAGS=' -O2'
%endif
%if "1" == "%{use_gnu3}"
        export CC=gcc
        export CFLAGS='-O2 -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE'
        export F77=g77
        export FFLAGS=' -O2'
%endif
# load mpi module
module load mpi/%{mpitag}-%{compilertag}

### build double
pushd double
./configure --prefix=%{prefix}   %{fftw2hacks} \
	--enable-mpi \
	 --with-pentium-timer  
%{__make} %{?_smp_mflags}
popd
pushd single
### build single
./configure --prefix=%{prefix}  %{fftw2hacks} \
	--enable-mpi \
	--with-pentium-timer  \
	--enable-type-prefix \
	--enable-float 
%{__make} %{?_smp_mflags}
popd
module rm mpi/%{mpitag}-%{compilertag}

%install
#all we care about is the library, the fftw-2 package should already be installed
pushd double
        /bin/mkdir -p  %{buildroot}/%{prefix}/lib
	/bin/cp mpi/.libs/*_mpi.a  %{buildroot}/%{prefix}/lib
popd
pushd single
        /bin/mkdir -p  %{buildroot}/%{prefix}/lib
	/bin/cp mpi/.libs/*_mpi.a  %{buildroot}/%{prefix}/lib
popd

%post
/sbin/ldconfig 2>/dev/null
#create the module file
cat <<EOF >%{modulepath}/%{name}

#%Module1.0#######################################################################
## fftw2 modulefile
##

prereq fftw2-%{compilertag}

proc ModulesHelp { } {
        puts stderr "\tThis module loads the appropriate environment for "
        puts stderr "\tusing fftw-%{mpitag}-%{compilertag} version %{version}.\n"
}

module-whatis "fftw2-%{mpitag}-%{compilertag} %{version}"
module-log error none

set fftw2root %{prefix}

if [ module-info mode load ] {
	setenv FFTW2_MPI_PATH \$fftw2root/lib/
	setenv FFTW2_INCLUDE \$fftw2root/include
        append-path INFOPATH \$fftw2root/info
}
if [ module-info mode remove ] {
	unsetenv FFTW2_MPI_PATH 
	unsetenv FFTW2_INCLUDE
        remove-path INFOPATH \$fftw2root/info
}
EOF

%postun
/sbin/ldconfig 2>/dev/null
/bin/rm %{modulepath}/%{name}


%clean
%{__rm} -rf %{buildroot}

%files
%defattr(-, root, root, 0755)
%{prefix}/lib/*.a

%changelog
